
#include <algorithm>

#include <GL/glu.h>

#include "glModel.h"

#define MODEL_DISPLAY_SCALE 10.0

GLfloat bodyColor[4]={1.0, 1.0, 1.0, 0.30};
GLfloat sharedColor[4]={0.88, 0.60, 0.06, 0.15};
GLfloat fixedColor[4]={1.00, 0.00, 0.00, 0.30};
GLfloat blackColor[4]={0.0, 0.0, 0.0, 1.0};
GLfloat errorColor[4]={1.0, 0.3, 0.3, 0.2};

void FeGLModel::glTransformation() {
}

struct EdgeDrawer: public unary_function<FeEdge *, void> {
  EdgeDrawer(bool status) { mStatus=status; };
  bool mStatus;
  void operator () (FeEdge *pe) {
    if (pe->isOK() == mStatus) {
      FeVertex v0=pe->v0();
      FeVertex v1=pe->v1();
      glVertex3d(MODEL_DISPLAY_SCALE*v0[0], MODEL_DISPLAY_SCALE*v0[1], MODEL_DISPLAY_SCALE*v0[2]);
      glVertex3d(MODEL_DISPLAY_SCALE*v1[0], MODEL_DISPLAY_SCALE*v1[1], MODEL_DISPLAY_SCALE*v1[2]);      
    }
  };
};

struct FaceDrawer: public unary_function<FeEdge *, void> {
  FaceDrawer(bool materialsOn) { mMaterialsFlag=materialsOn; }
  bool mMaterialsFlag;
  void operator () (FeFace *pf) {
    if (mMaterialsFlag) {
      FeVector norm=pf->normal();
      glNormal3d(norm.x(), norm.y(), norm.z());
      if (pf->isOK()) {
	if (pf->oppositeFace()) {
	  glMaterialfv(GL_FRONT_AND_BACK, GL_AMBIENT_AND_DIFFUSE, sharedColor);
	} else {
	  if (pf->fixed()) {
	    glMaterialfv(GL_FRONT_AND_BACK, GL_AMBIENT_AND_DIFFUSE, fixedColor);
	  } else {
	    glMaterialfv(GL_FRONT_AND_BACK, GL_AMBIENT_AND_DIFFUSE, bodyColor);
	  }
	}
      } else {
	glMaterialfv(GL_FRONT_AND_BACK, GL_AMBIENT_AND_DIFFUSE, errorColor);
      }
      GLfloat actShininess=1.0;
      glMaterialfv(GL_FRONT, GL_SHININESS, &actShininess);
    } 

    glBegin(GL_POLYGON);
    FeVertex pv=pf->firstVertex();
    try {
      while (1) {
	glVertex3d(MODEL_DISPLAY_SCALE*pv[0], MODEL_DISPLAY_SCALE*pv[1], MODEL_DISPLAY_SCALE*pv[2]);
	pv=pf->nextVertex();
      }
    } catch (FeListEnd &) {}
    glEnd();    
  };
};

void FeGLModel::glDraw(GLViewport *pv) {
  switch (pv->mode) {
    case vpWireframe: {      
      glDisable(GL_DEPTH_TEST);

      glLineWidth(2.0);

      // Draw edges with status OK
      glColor3f(bodyColor[0], bodyColor[1], bodyColor[2]);
      glBegin(GL_LINES);
      EdgeDrawer dr1(true);
      for_each(mvpEdges.begin(), mvpEdges.end(), dr1);
      glEnd();

      // Draw bad edges
      glColor3f(errorColor[0], errorColor[1], errorColor[2]);
      glBegin(GL_LINES);
      EdgeDrawer dr2(false);
      for_each(mvpEdges.begin(), mvpEdges.end(), dr2);
      glEnd();
    } break;
    case vpHiddenline: {
      glDisable(GL_DEPTH_TEST);

      glLineWidth(2.0);
      glEnable(GL_LINE_STIPPLE);
      glLineStipple(4, 0xAAAA);

      glColor3f(bodyColor[0], bodyColor[1], bodyColor[2]);
      glBegin(GL_LINES);
      EdgeDrawer edr1(true);
      for_each(mvpEdges.begin(), mvpEdges.end(), edr1);
      glEnd();

      glColor3f(errorColor[0], errorColor[1], errorColor[2]);
      glBegin(GL_LINES);
      EdgeDrawer edr2(false);
      for_each(mvpEdges.begin(), mvpEdges.end(), edr2);
      glEnd();

      glDisable(GL_LINE_STIPPLE);
      
      glEnable(GL_DEPTH_TEST);
      glDepthFunc(GL_LEQUAL);

      glEnable(GL_POLYGON_OFFSET_FILL);
      glPolygonOffset(1.0, 1.0);

      glColorMask(GL_FALSE, GL_FALSE, GL_FALSE, GL_FALSE);
      FaceDrawer fdr1(false);
      for_each(mvpFaces.begin(), mvpFaces.end(), fdr1);
      glColorMask(GL_TRUE, GL_TRUE, GL_TRUE, GL_TRUE);            

      glDisable(GL_POLYGON_OFFSET_FILL);

      glLineWidth(2.0);

      glColor3f(bodyColor[0], bodyColor[1], bodyColor[2]);
      glBegin(GL_LINES);
      for_each(mvpEdges.begin(), mvpEdges.end(), edr1);
      glEnd();

      glColor3f(errorColor[0], errorColor[1], errorColor[2]);
      glBegin(GL_LINES);
      for_each(mvpEdges.begin(), mvpEdges.end(), edr2);
      glEnd();

    } break;
    case vpFlat:
    case vpSmooth:  
      glDisable(GL_DEPTH_TEST);
      glEnable(GL_BLEND);
      //      glEnable(GL_CULL_FACE);
      //      glCullFace(GL_BACK);
      glFrontFace(GL_CCW);
      glBlendFunc (GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
      FaceDrawer dr1(true);
      for_each(mvpFaces.begin(), mvpFaces.end(), dr1);
      glDisable(GL_BLEND);
      glEnable(GL_DEPTH_TEST);
      break;
  }
}

void FeGLModel::glDrawBound(GLViewport *pv) {
}

void drawFaceNormal(FeFace *pf) {
  FeVector pc=pf->center();
  FeVector pn=pf->normal();
  pc=pc*MODEL_DISPLAY_SCALE;
  FeVector nd=pn*7.0;
  FeVector v2=pc + nd;

  if (pf->oppositeFace()) {
    glColor3f(0.88, 0.60, 0.06);
  } else {
    if (pf->fixed()) {
      glColor3f(0.8, 0.2, 0.2);
    } else {
      glColor3f(1.0, 1.0, 0.3);
    }
  }

  glBegin(GL_LINES);
  glVertex3d(pc.x(), pc.y(), pc.z());
  glVertex3d(v2.x(), v2.y(), v2.z());
  glEnd();

  glMatrixMode(GL_MODELVIEW);
  glPushMatrix();
  glTranslated(pc.x(), pc.y(), pc.z());
  GLUquadric *pq=gluNewQuadric();
  gluQuadricDrawStyle(pq, GLU_LINE);
  gluSphere(pq, 1.0, 4, 2);
  if (pf->oppositeFace()) {
    glTranslated(nd.x(), nd.y(), nd.z());
    gluSphere(pq, 0.5, 4, 2);
  }
  if (pf->fixed()) {
    glTranslated(nd.x(), nd.y(), nd.z());
    gluSphere(pq, 2.0, 4, 2);    
  }
  glPopMatrix();
}

void FeGLModel::glDrawNormals(GLViewport *pv) {
  glDisable(GL_DEPTH_TEST);
  glEnable(GL_LINE_STIPPLE);
  glLineStipple(2, 0xAAAA);
  for_each(mvpFaces.begin(), mvpFaces.end(), drawFaceNormal);

  glEnable(GL_DEPTH_TEST);
  glDisable(GL_LINE_STIPPLE);
  for_each(mvpFaces.begin(), mvpFaces.end(), drawFaceNormal);
}

/*--- End of file ---*/
