
/*
  Basic class shared by all classes in libFailure
*/

#ifndef _FE_BASIC_H
#define _FE_BASIC_H

#include <stdlib.h>
#include <assert.h>
#include <iostream>
#include <vector>
#include <string>

//#ifdef _OLD_GCC
//#include <hash_set>
//#else
//#include <ext/hash_set>
//#endif

#define FE_FLOAT_PREC 1e-5

#define FE_INFINITY 1e20

#ifndef FloatPrec
#define FloatPrec FE_FLOAT_PREC
#endif

#ifndef GRAVITY
//#define GRAVITY 9.81
#define GRAVITY 10
#endif

#ifndef PI
#define PI M_PI
#endif

#define DEFAULT_DEBUG_LEVEL 3

extern double feDebugLevel;

typedef size_t ID;

using namespace std;

#ifndef _OLD_GCC
using namespace __gnu_cxx;
#endif

class FeBasic { 
 private:
  ID mvID;
  bool mvIsOK;
 public:
  FeBasic();
  virtual ~FeBasic();
  
  ID id();
  string idS();

  void isOK(bool value);
  bool isOK();

  virtual void checkTopology();

  virtual void writeToFile(ostream &os);

  virtual void info(ostream &os);  
};

#define DEBUG(level, what) if (feDebugLevel >= level) { what; } 

//template <class T> struct KillFromHash {
//  KillFromHash(hash_set<T *, hash<T *> > &ph): actHash(ph) {};
//  hash_set<T *, hash<T *> > actHash;
//  void operator () (T *pt) { delete pt; };
//};

template <class T> struct KillFromVector {
  KillFromVector(vector<T *> &pv): actVector(pv) {};
  vector<T *> actVector;
  void operator () (T *pt) { delete pt; };
};

#ifndef Sqr
#define Sqr(x) ((x) * (x))
#endif

#ifndef Det3
#define Det3(a,b,c, d,e,f, g,h,i) ((a)*(e)*(i) + (d)*(h)*(c) + (g)*(b)*(f) - (g)*(e)*(c) - (d)*(b)*(i) - (a)*(h)*(f))
#endif

#endif
