
#ifndef _FE_BODY_H
#define _FE_BODY_H

#include <vector>

#include "basic.h"
#include "vector.h"

class FeFace;
class FeModel;

class FeBody: public FeBasic {
  friend class FeModel;
 protected:
  FeModel *mvpModel;

 private:

  FeVector mvInitialVelocity;
  FeVector mvVelocity;

  vector<FeFace *> mvpFaces;  // List of all faces

  FeVector mvCenter;
  bool mvCenterValid;  

  double mvDensity;

  double mvVolume;
  double mvPercolatedVolume;
  double mvWeight;
  
  vector<FeVector> mvFrictionForces;
  vector<FeVector> mvCohesionForces;
  FeVector mvFrictionForcesSum;
  FeVector mvCohesionForcesSum;

  FeVector mvExternalForces;
  
  FeVector mvEquilibriumForceDirection;
  
  int mvNumber;

 public:
  FeBody();
  virtual ~FeBody();

  FeModel *model();

  void initialVelocity(FeVector &pv);
  void initialVelocity(double pv[]);
  FeVector *initialVelocity();

  void velocity(FeVector &pv);
  FeVector velocity();

  double density(double d=0.0);

  void addFace(FeFace *);  
  
  vector<FeFace *>::iterator firstFace();
  vector<FeFace *>::iterator lastFace();
  
  virtual void checkTopology();

  int number(int newNumber=0);

  FeVector center();
  double volume();
  double percolatedVolume();
  double weight();

  FeVector& externalForces(FeVector &);
  FeVector& externalForces();

  FeVector& equilibriumForceDirection(FeVector &);
  FeVector& equilibriumForceDirection();

  void invalidatePrecomputedValues();

  void calculateCohesionForces();  

  FeVector& cohesionForcesSum();

  virtual void writeToFile(ostream &os);

  virtual void info(ostream &os);    
};

#endif

