
#include <algorithm>

#include "basic.h"
#include "edge.h"
#include "vertex.h"
#include "face.h"
#include "wing.h"

FeEdge::FeEdge(FeVertex *a, FeVertex *b) {
  assert(a && b);

  mvpVertices[0]=a;
  mvpVertices[1]=b;

  a->addEdge(this);
  b->addEdge(this);
}

FeEdge::~FeEdge() {
  DEBUG(10, cout << "\t\tDestroying edge " << id() << endl);  
  
  KillFromVector<FeWing> pk1(mvpWings);
  for_each(mvpWings.begin(), mvpWings.end(), pk1);
  mvpWings.clear();

  mvpVertices[0]->removeEdge(this);
  mvpVertices[1]->removeEdge(this);
}

FeVertex &FeEdge::operator [] (int a) {
  assert(a >= 0 && a < 2);
  assert(mvpVertices[a]);
  return *mvpVertices[a];
}

FeVertex &FeEdge::v0() {
  return *mvpVertices[0];
}

FeVertex &FeEdge::v1() {
  return *mvpVertices[1];
}

void FeEdge::addWing(FeWing *pw) {
  assert(pw);

  mvpWings.push_back(pw);
}

FeWing *FeEdge::createWing(const FeVertex *v0, const FeVertex *v1) {
  if (v0 == mvpVertices[0] && v1 == mvpVertices[1]) return new FeWing(this, 0);
  if (v0 == mvpVertices[1] && v1 == mvpVertices[0]) return new FeWing(this, 1);
  return NULL;
}

void FeEdge::invalidatePrecomputedValues() {
  vector<FeWing *>::iterator actW=mvpWings.begin();
  while (actW != mvpWings.end()) {
    (*actW)->face()->invalidatePrecomputedValues();
    actW++;
  }
}

void FeEdge::checkTopology() {
  DEBUG(9, cout << "Checking edge " << id() << " topology" << endl);

  // All bodies should be closed, so all edges should have even number of wings
  int numF=mvpWings.size();
  if (numF != 2*(numF/2)) isOK(false);
}

/*--- End of file ---*/

