
/*
  Written in March 2002 by Vaclav Cermak
*/

/*
  Convex face class
*/

#ifndef _FE_FACE_H
#define _FE_FACE_H

#include <vector>
#include <list>

#include "basic.h"
#include "vector.h"

class FeVertex;
class FeWing;
class FeBody;

#define DEFAULT_FRICTION_ANGLE 30.0
//#define DEFAULT_COHESION 20
#define DEFAULT_COHESION 0

class FeFace: public FeBasic {
 private:
  vector<FeWing *> mvpWings; // List of all wings in CCW order
  FeBody *mvpBody;
  FeFace *mvpOppositeFace;

  int mvNumber;

  bool mvFixed;
  double mvFrictionAngle;
  double mvCohesion;

  vector<FeWing *>::iterator mvpActWing;

  FeVector mvNormal;

  FeVector mvCenter;
  bool mvCenterValid;

  double mvArea;
  double mvPercolatedArea;

  FeVector mvFrictionForce;

#ifndef _THREAD_SAFE
  static FeVertex *mvpLastVertex;
#else
  FeVertex *mvpLastVertex;
#endif

 public:
  FeFace();
  virtual ~FeFace();

  string eqIdentifier();

  int number(int newNumber=0);

  void fixed(bool value);
  bool fixed();

  void addVertex(FeVertex *pv);
  void finish();              // Finish face and check topology

  FeVertex &firstVertex();
  FeVertex &nextVertex();

  virtual void checkTopology();
  
  void setBody(FeBody *pb);
  FeBody *body();

  void setOppositeFace(FeFace *pf);
  FeFace *oppositeFace();

  void frictionAngle(double angle);
  double frictionAngle();
  double tanFrictionAngle();
  double sinFrictionAngle();
  double cosFrictionAngle();

  void cohesion(double cohesion);
  double cohesion();

  FeVector normal();
  FeVector center();
  double area();
  double percolatedArea();

  FeVector cohesionForce();

  FeVector frictionForceDir();

  double vertexDist(FeVector &pv);

  void invalidatePrecomputedValues();

  virtual void writeToFile(ostream &os);

  virtual void info(ostream &os);
  
};

#endif
