
#ifndef _FE_MODEL_H
#define _FE_MODEL_H

#include <functional>

#include "basic.h"
#include "vertex.h"
#include "edge.h"
#include "face.h"
#include "body.h"

//////////////////////////////////////
//
// Macros defining work of iteration routine
//

#define CHECK_IF_NEW_EQ_FORCE_IS_BETTER(newForce, oldForce) ( newForce < oldForce )

//
//////////////////////////////////////

#define VERTEX_HASH_KOEF 10000

#define MAX_PRISM_POINTS 256

class FeEdge;
class FeFace;

class FeModel: public FeBasic {
 protected:
  vector<FeVertex *> mvpVertices;  // All vertices used in model
  vector<FeEdge *> mvpEdges;       // All edges used in model
  vector<FeFace *> mvpFaces;       // All faces used in model
  vector<FeBody *> mvpBodies;      // All bodies used in model

  FeBody *mvpActBody;              // Body which is already in creation state
  FeFace *mvpActFace;              // Face which is already in creation state

  FeVector mvFrictionForcesSum;
  FeVector mvCohesionForcesSum;
  FeVector mvGravitationalForcesSum;

  vector<int> mvPertubatedVertices;    // Indexes of vertices which should be pertubated (zero based)
  
  double mvWaterLevel;

  double mvMinimalEFValue;             // Actual minimal equlibrium force value
  vector<FeVector> mvMinimalEFVCoords; // Coordinates of vertices coresponding to actual minimal equlibrium force
  int mvMinimalEFNumIt;

#ifdef REMEMBER_LAST_ITERATION
  vector<FeVector> mvLastVCoords;
#endif

#ifdef REMEMBER_ALL_ITERATIONS
  vector<vector <FeVector> > mvAllVCoords;
#endif

  FeBody *addTetrahedron(FeVertex *, FeVertex *, FeVertex *, FeVertex *, bool *);

  vector<FeVector> copyVerticesCoords();

 public:
  FeModel();
  virtual ~FeModel();

  int addVertex(FeVector &);
  FeVertex *getVertex(int index);
  void changeVertex(int index, FeVector &newVertex); // Change vertex geometry
  void changeVertex(int index, double x, double y, double z); // Change vertex geometry

  void setPertubatedVertices(const int number, const int *indexes); // Set pertubated vertices, indexes starts from one!

  FeEdge *createEdge(FeVertex *v0, FeVertex *v1);

  FeBody *startBody();             // Starts building of body 
  void endBody();                  // End building of body, performs topological check

  FeFace *faceStart();                                // Starts building of face
  FeVertex *faceAddVertex(int index);                     // Add vertex in face
  void faceEnd();                                     // Ends building of face
  
  FeFace *lookupBodyBodyFace(int bodyNum1, int bodyNum2);
  FeFace *lookupFace(int bodyNum, int faceNum);

  double waterLevel(double newWaterLevel=FE_INFINITY);

  //  FeBody *addTetrahedron(double points[4][3], bool *stateList);
  //  FeBody *addTetrahedron(const double p1[3], const double p2[3], const double p3[3], const double p4[3], bool *stateList);
  //  FeBody *addTetrahedron(const FeVector &p1, const FeVector &p2, const FeVector &p3, const FeVector &p4, bool *stateList);
  //  FeBody *addPrism(unsigned short pNum, double pPoints[MAX_PRISM_POINTS][3], double pHgt, bool *stateList);
  //  FeBody *addPyramid(unsigned short pNum, double pPoints[MAX_PRISM_POINTS][3], double pHgt, bool *stateList);

  void detectSharedFaces();

  void solveVelocities();
  //  FeVector& solveFrictionForces();
  FeVector& calculateCohesionForces();
  FeVector& calculateGravitationalForces();
 
  double findEquilibrium();

  bool pertubVertices(int iterationNumber, bool lastIterationOK); // Change vertices, return value false means stop calculation

  double findMinimalEqulibriumForce(int maxIterations);

  virtual void checkTopology();  

  virtual void writeToFile(ostream &os);

  virtual void info(ostream &os);
};

#endif

