
/*
  Written in March 2002 by Vaclav Cermak
*/

/*
  Three dimensional vector class
*/

#ifndef _VECTOR_H
#define _VECTOR_H

#include <iostream>

#include "basic.h"

class FeVertex;

class FeVector: public FeBasic {
 protected:
  double mvA[3];

 public:
  FeVector();
  FeVector(double x, double y, double z);
  FeVector(double *a);
  FeVector(const FeVector &v);
  FeVector(const FeVector *pv);

  double& operator [] (int i);

  FeVector &operator = (const FeVector &);
  FeVector &operator += (const FeVector &);
  FeVector &operator -= (const FeVector &);

  FeVector operator + (const FeVector &) const;
  FeVector operator - (const FeVector &) const;
  FeVector operator * (const double) const;
  FeVector operator / (const double) const;

  double operator ^ (const FeVector &) const;    // Dot product
  FeVector operator * (const FeVector &) const;

  double sqrLength();
  double length();
  void normalize();

  double x();
  double y();
  double z();

  friend ostream& operator << (ostream &os, const FeVector &pv);  
  friend ostream& operator << (ostream &os, const FeVector *pva);  

  friend FeVector operator + (FeVertex &v0, FeVertex &v1);
  friend FeVector operator - (FeVertex &v0, FeVertex &v1);
  
  friend FeVector operator * (double, FeVector &);

  virtual void writeToFile(ostream &os);
};

static FeVector FeZeroVector(0.0, 0.0, 0.0);

#endif
