
#include <algorithm>

#include "vertex.h"
#include "edge.h"
#include "wing.h"
#include "model.h"

FeVertex::FeVertex(): FeVector(0, 0, 0) {
}

FeVertex::FeVertex(double x, double y, double z): FeVector(x, y, z) {
}

FeVertex::FeVertex(const FeVector &v): FeVector(v) {
}

FeVertex::~FeVertex() {
  DEBUG(10, cout << "\t\tDestroying vertex " << id() << endl);
}

void FeVertex::addEdge(FeEdge *pe) {
  assert(pe);
  mvpEdges.push_front(pe);
}

void FeVertex::removeEdge(FeEdge *pe) {
  assert(pe);
  list<FeEdge *>::iterator res=find(mvpEdges.begin(), mvpEdges.end(), pe);
  assert(res != mvpEdges.end());
  mvpEdges.erase(res);
}

struct NewWing: public unary_function<FeEdge *, bool> {
  NewWing(const FeVertex *v0, const FeVertex *v1, FeWing **wingRef) { mV0=v0; mV1=v1; newWing=wingRef; *newWing=NULL; };
  const FeVertex *mV0, *mV1;
  FeWing **newWing;
  bool operator () (FeEdge *pe) {
    *newWing=pe->createWing(mV0, mV1);
    return *newWing ? true : false;
  };
};

FeWing *FeVertex::getWingTo(FeVertex *pv) {
  FeWing *newWing;
  NewWing finder(this, pv, &newWing);
  find_if(mvpEdges.begin(), mvpEdges.end(), finder);

  if (newWing) {
    return newWing;
  } else {
    FeEdge *ne=mvpModel->createEdge(this, pv);
    return ne->createWing(this, pv);
  }
}

void FeVertex::invalidatePrecomputedValues() {
  list<FeEdge *>::iterator actE=mvpEdges.begin();
  while (actE != mvpEdges.end()) {
    (*actE)->invalidatePrecomputedValues();

    actE++;
  }
}

/*--- End of file ---*/

