
/*
  Oook geometry library

  Written in 1998 by Vaclav Cermak (disnel@linux.itam.cas.cz)

  Matrix 4x4 used as transfromation matrix
  If initialized, [0][0], [1][1], [2][2] and [3][3] items are set to 1.0

*/

#ifndef _OGL_MATRIX4_H
#define _OGL_MATRIX4_H

#include "OGLBasic.h"
#include "OGLVector3.h"
#include "OGLVector4.h"

class OGLMatrix4 {
protected:
  OGLFloat a[16];

public:
  OGLMatrix4(OGLBool initialize=1);
  OGLMatrix4(const OGLMatrix4 &);
  
  static OGLMatrix4 createIdentity();
  static OGLMatrix4 createTranslation(const OGLVector3 &);
  static OGLMatrix4 createRotationDeg(const OGLVector3 &);
  static OGLMatrix4 createRotationRad(const OGLVector3 &);
  static OGLMatrix4 createScale(const OGLVector3 &);
  static OGLMatrix4 createMatrix(const OGLFloat [16]);

  OGLMatrix4 operator * (const OGLMatrix4 &) const;
  OGLVector4 operator * (const OGLVector4 &) const;
  OGLVector3 transformVector(const OGLVector3 &) const;
  OGLVector3 transformVertex(const OGLVector3 &) const;
  OGLVector4 transformVertex(const OGLVector4 &) const;

  OGLFloat operator () (OGLInt, OGLInt) const;
  OGLFloat &operator () (OGLInt, OGLInt);

  friend ostream &operator << (ostream &, const OGLMatrix4 &);

#ifdef _USE_GL_
  void loadGLMatrix();
  void multGLMatrix();
#endif
};

#endif
