/*
  Oook geometry library

  Written in 1998 by Vaclav Cermak (disnel@linux.itam.cas.cz)

  Two dimensional vector code
*/

#include "OGLVector2.h"

OGLVector2 OGLv2O(0,0);
OGLVector2 OGLv2X(1,0);
OGLVector2 OGLv2Y(0,1);

OGLVector2::OGLVector2(const OGLFloat x, const OGLFloat y) {
  a[0]=x; a[1]=y;
}

OGLVector2::OGLVector2(const OGLVector2 &v) {
  a[0]=v.a[0]; a[1]=v.a[1];
}

OGLVector2 &OGLVector2::operator = (const OGLVector2 &v) {
  if (&v != this) { a[0]=v.a[0]; a[1]=v.a[1]; }
  return *this;
}

OGLVector2 &OGLVector2::operator += (const OGLVector2 &v) {
  a[0]+=v.a[0];
  a[1]+=v.a[1];
  return *this;
}

OGLVector2 &OGLVector2::operator -= (const OGLVector2 &v) {
  a[0]-=v.a[0];
  a[1]-=v.a[1];
  return *this;
}

OGLVector2 OGLVector2::operator + (const OGLVector2 &v) const {
  return OGLVector2(a[0]+v.a[0], a[1]+v.a[1]);
}

OGLVector2 OGLVector2::operator - (const OGLVector2 &v) const {
  return OGLVector2(a[0]-v.a[0], a[1]-v.a[1]);
}

OGLVector2 OGLVector2::operator * (const OGLFloat k) const {
  return OGLVector2(k*a[0], k*a[1]);
}

OGLVector2 OGLVector2::operator / (const OGLFloat k) const {
  return OGLVector2(a[0]/k, a[1]/k);
}

OGLFloat &OGLVector2::operator [] (int i) {
  return a[i%2];
}

OGLFloat OGLVector2::operator [] (int i) const {
  return a[i%2];
}

void OGLVector2::selfNormalize() {
  OGLFloat plen=len();
  a[0]/=plen;
  a[1]/=plen;
}

OGLVector2 OGLVector2::normalize() const {
  OGLFloat plen=len();
  return OGLVector2(a[0]/plen, a[1]/plen);
}

OGLFloat OGLVector2::sqrLen() const {
  return a[0]*a[0] + a[1]*a[1];
}

OGLFloat OGLVector2::len() const {
  return sqrt(a[0]*a[0] + a[1]*a[1]);
}

OGLFloat OGLVector2::operator ^ (const OGLVector2 &v) const {
  return a[0]*v.a[0] + a[1]*v.a[1];
}

ostream &operator << (ostream &os, const OGLVector2 &v) {
  os << "<" << v.a[0] << ", " << v.a[1] << ">";
  return os;
}

#ifdef _USE_GL_
void OGLVector2::glVertex() {
#ifdef _USE_GL_DOUBLE_PRECISION_
  glVertex2d(a[0], a[1]);
#else
  glVertex2f(a[0], a[1]);
#endif  
}
#endif

/*--- End of file ---*/
