/*
  Oook geometry library

  Written in 1998 by Vaclav Cermak (disnel@linux.itam.cas.cz)

  Three dimensional vector code
*/

#include "OGLVector3.h"

OGLVector3 OGLv3O(0,0,0);
OGLVector3 OGLv3X(1,0,0);
OGLVector3 OGLv3Y(0,1,0);
OGLVector3 OGLv3Z(0,0,1);

OGLVector3::OGLVector3(const OGLFloat x, const OGLFloat y, const OGLFloat z) {
  a[0]=x; a[1]=y; a[2]=z;
}

OGLVector3::OGLVector3(const OGLVector2 &v) {
  a[0]=v[0]; a[1]=v[1]; a[2]=0.0;
}

OGLVector3::OGLVector3(const OGLVector3 &v) {
  a[0]=v.a[0]; a[1]=v.a[1]; a[2]=v.a[2];
}

OGLVector3 &OGLVector3::operator = (const OGLVector3 &v) {
  if (&v != this) { a[0]=v.a[0]; a[1]=v.a[1]; a[2]=v.a[2]; }
  return *this;
}

OGLVector3 &OGLVector3::operator += (const OGLVector3 &v) {
  a[0]+=v.a[0];
  a[1]+=v.a[1];
  a[2]+=v.a[2];
  return *this;
}

OGLVector3 &OGLVector3::operator -= (const OGLVector3 &v) {
  a[0]-=v.a[0];
  a[1]-=v.a[1];
  a[2]-=v.a[2];
  return *this;
}

OGLVector3 OGLVector3::operator + (const OGLVector3 &v) const {
  return OGLVector3(a[0]+v.a[0], a[1]+v.a[1], a[2]+v.a[2]);
}

OGLVector3 OGLVector3::operator - (const OGLVector3 &v) const {
  return OGLVector3(a[0]-v.a[0], a[1]-v.a[1], a[2]-v.a[2]);
}

OGLVector3 OGLVector3::operator * (const OGLFloat k) const {
  return OGLVector3(k*a[0], k*a[1], k*a[2]);
}

OGLVector3 OGLVector3::operator / (const OGLFloat k) const {
  return OGLVector3(a[0]/k, a[1]/k, a[2]/k);
}

OGLFloat &OGLVector3::operator [] (int i) {
  return a[i%3];
}

OGLFloat OGLVector3::operator [] (int i) const {
  return a[i%3];
}

void OGLVector3::selfNormalize() {
  OGLFloat plen=len();
  a[0]/=plen;
  a[1]/=plen;
  a[2]/=plen;
}

void OGLVector3::setLen(OGLFloat newLen) {
  selfNormalize();
  a[0]*=newLen;
  a[1]*=newLen;
  a[2]*=newLen;
}

OGLVector3 OGLVector3::normalize() const {
  OGLFloat plen=len();
  return OGLVector3(a[0]/plen, a[1]/plen, a[2]/plen);
}

OGLFloat OGLVector3::sqrLen() const {
  return a[0]*a[0] + a[1]*a[1] + a[2]*a[2];
}

OGLFloat OGLVector3::len() const {
  return sqrt(a[0]*a[0] + a[1]*a[1] + a[2]*a[2]);
}

OGLFloat OGLVector3::operator ^ (const OGLVector3 &v) const {
  return a[0]*v.a[0] + a[1]*v.a[1] + a[2]*v.a[2];
}

OGLVector3 OGLVector3::operator * (const OGLVector3 &v) const {
  return OGLVector3(Det2(a[1], a[2], v.a[1], v.a[2]),
		    -Det2(a[0], a[2], v.a[0], v.a[2]),
		    Det2(a[0], a[1], v.a[0], v.a[1]));
}

ostream &operator << (ostream &os, const OGLVector3 &v) {
  os << "<" << v.a[0] << ", " << v.a[1] << ", " << v.a[2] << ">";
  return os;
}

ostream &operator << (ostream &os, const OGLVector3 *v) {
  os << "<" << v->a[0] << ", " << v->a[1] << ", " << v->a[2] << ">";
  return os;
}

#ifdef _USE_GL_

void OGLVector3::glVertex() {
#ifdef _USE_GL_DOUBLE_PRECISION_
  glVertex3d(a[0], a[1], a[2]);
#else
  glVertex3f(a[0], a[1], a[2]);
#endif  
}

void OGLVector3::glNormal() {
#ifdef _USE_GL_DOUBLE_PRECISION_
  glNormal3d(a[0], a[1], a[2]);
#else
  glNormal3f(a[0], a[1], a[2]);
#endif  
}

#endif

/*--- End of file ---*/
 
