
/*
  Oook geometry library
  
  Written in 1998 by Vaclav Cermak (disnel@linux.itam.cas.cz)
  
  Four dimensional vector code
*/

#include "OGLVector4.h"

OGLVector4::OGLVector4(const OGLFloat x, const OGLFloat y, const OGLFloat z, const OGLFloat d) {
  a[0]=x; a[1]=y; a[2]=z; a[3]=d;
}

OGLVector4::OGLVector4(const OGLVector2 &v) {
  a[0]=v[0]; a[1]=v[1]; a[2]=0.0; a[3]=0.0;
}

OGLVector4::OGLVector4(const OGLVector3 &v) {
  a[0]=v[0]; a[1]=v[1]; a[2]=v[2]; a[3]=0.0;
}

OGLVector4::OGLVector4(const OGLVector4 &v) {
  a[0]=v.a[0]; a[1]=v.a[1]; a[2]=v.a[2]; a[3]=v.a[4];
}

OGLVector4 &OGLVector4::operator = (const OGLVector4 &v) {
  if (&v != this) { a[0]=v.a[0]; a[1]=v.a[1]; a[2]=v.a[2]; a[3]=v.a[3]; }
  return *this;
}

OGLVector4 &OGLVector4::operator += (const OGLVector4 &v) {
  a[0]+=v.a[0];
  a[1]+=v.a[1];
  a[2]+=v.a[2];  
  a[3]+=v.a[3];  
  return *this;
}

OGLVector4 &OGLVector4::operator -= (const OGLVector4 &v) {
  a[0]-=v.a[0];
  a[1]-=v.a[1];
  a[2]-=v.a[2];
  a[3]-=v.a[3];
  return *this;
}

OGLVector4 OGLVector4::operator + (const OGLVector4 &v) const {
  return OGLVector4(a[0]+v.a[0], a[1]+v.a[1], a[2]+v.a[2], a[3]+v.a[3]);
}

OGLVector4 OGLVector4::operator - (const OGLVector4 &v) const {
  return OGLVector4(a[0]-v.a[0], a[1]-v.a[1], a[2]-v.a[2], a[3]-v.a[3]);
}

OGLVector4 OGLVector4::operator * (const OGLFloat k) const {
  return OGLVector4(k*a[0], k*a[1], k*a[2], k*a[3]);
}

OGLVector4 OGLVector4::operator / (const OGLFloat k) const {
  return OGLVector4(a[0]/k, a[1]/k, a[2]/k, a[3]/k);
}

OGLFloat &OGLVector4::operator [] (int i) {
  return a[i%4];
}

OGLFloat OGLVector4::operator [] (int i) const {
  return a[i%4];
}

void OGLVector4::selfNormalize() {
  OGLFloat plen=len();
  a[0]/=plen;
  a[1]/=plen;
  a[2]/=plen;
  a[3]/=plen;
}

OGLVector4 OGLVector4::normalize() const {
  OGLFloat plen=len();
  return OGLVector4(a[0]/plen, a[1]/plen, a[2]/plen, a[3]/plen);
}

OGLFloat OGLVector4::sqrLen() const {
  return a[0]*a[0] + a[1]*a[1] + a[2]*a[2] + a[3]*a[3];
}

OGLFloat OGLVector4::len() const {
  return sqrt(a[0]*a[0] + a[1]*a[1] + a[2]*a[2] + a[3]*a[3]);
}

OGLFloat OGLVector4::operator ^ (const OGLVector4 &v) const {
  return a[0]*v.a[0] + a[1]*v.a[1] + a[2]*v.a[2] + a[3]*v.a[3];
}

OGLVector4 OGLVector4::operator * (const OGLVector4 &v) const {
  if (a[3] != 0.0 || v.a[3] != 0.0) throw "Cannot perform vector multiplication on 4D vertices";
  return OGLVector3(Det2(a[1], a[2], v.a[1], v.a[2]),
		    -Det2(a[0], a[2], v.a[0], v.a[2]),
		    Det2(a[0], a[1], v.a[0], v.a[1]));
}

OGLVector3 OGLVector4::createVertex() {
  if (fabs(a[3]) < FloatPrec) throw "Cannot create vertex from vector";
  return OGLVector3(a[0]/a[3], a[1]/a[3], a[2]/a[3]);
}

ostream &operator << (ostream &os, const OGLVector4 &v) {
  os << "<" << v.a[0] << ", " << v.a[1] << ", " << v.a[2] << ", " << v.a[3] << ">";
  return os;
}

#ifdef _USE_GL_
void OGLVector4::glVertex() {
  //  cout << "glVertex(" << a[0] << ", " << a[1] << ", " << a[2] << ", " << a[3] << ")\n";
#ifdef _USE_GL_DOUBLE_PRECISION_
  if (a[3] >= 0.0) glVertex4d(a[0], a[1], a[2], a[3]);
  else glVertex4d(-a[0], -a[1], -a[2], -a[3]);
#else
  if (a[3] >= 0.0) glVertex4f(a[0], a[1], a[2], a[3]);
  else glVertex4f(-a[0], -a[1], -a[2], -a[3]);
#endif  
}
#endif

/*--- End of file ---*/
