

#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <GL/gl.h>
#include <GL/glu.h>

#include "viewport.h"

Viewport::Viewport(int _number, viewportType _type, viewportRenderMode _mode) {
  number=_number;
  type=_type; mode=_mode;
  motionMode=vpNoMotion;

  drawCenterFlag=false;
  drawGridFlag=true;
  drawCamerasFlag=drawLightsFlag=drawObjectsFlag=true;
  drawBoundsFlag=false;
  drawNormalsFlag=false;  

  defaultView();
}

Viewport::~Viewport() {
}

void Viewport::setType(viewportType newType) {
  type=newType;
  defaultView();
  redraw();
}

void Viewport::setRenderMode(viewportRenderMode newMode) {
  mode=newMode;
  redraw();
}

void Viewport::setMotionMode(viewportMotionMode newMode) {
  motionMode=newMode;
}

viewportMotionMode Viewport::getMotionMode() {
  return motionMode;
}

void Viewport::setRenderingArea(unsigned int x, unsigned int y, unsigned int w, unsigned int h) {
  rcX=x; rcY=y;
  rcW=w; rcH=h;
}

void Viewport::defaultView() {
  scale=1.0;
  switch (type) {
    case vpTop:
      eye[0]=0; eye[1]=1; eye[2]=0;
      center[0]=0; center[1]=0; center[2]=0;
      up[0]=0; up[1]=0; up[2]=-1;
      zInvert=-1;
      break;
    case vpFront:
      eye[0]=0; eye[1]=0; eye[2]=1;
      center[0]=0; center[1]=0; center[2]=0;
      up[0]=0; up[1]=1; up[2]=0;
      zInvert=1;
      break;
    case vpRight:
      eye[0]=1; eye[1]=0; eye[2]=0;
      center[0]=0; center[1]=0; center[2]=0;
      up[0]=0; up[1]=1; up[2]=0;
      zInvert=-1;
      break;
    case vpAxo:
    case vpPersp:
      eye[0]=-100; eye[1]=-100; eye[2]=100;
      center[0]=0; center[1]=0; center[2]=0;
      //      up[0]=0; up[1]=1; up[2]=0;
      up[0]=0; up[1]=0; up[2]=1;
      fov=45;
      zInvert=1;
      break;
  }
}

#define PREPARE_ROTATION \
  OGLVector3 e(center[0] - eye[0], \
	       center[1] - eye[1], \
	       center[2] - eye[2]); \
  OGLFloat oldLen=e.len(); \
   \
  OGLVector3 u(-Det2(e[1], e[2], up[1], up[2]), \
	       Det2(e[0], e[2], up[0], up[2]), \
	       -Det2(e[0], e[1], up[0], up[1])); \
  u.selfNormalize(); \
  OGLVector3 v(-Det2(u[1], u[2], up[1], up[2]), \
	       Det2(u[0], u[2], up[0], up[2]), \
	       -Det2(u[0], u[1], up[0], up[1])); \
  v.selfNormalize(); \
  OGLVector3 w(up[0], up[1], up[2]); \
  w.selfNormalize(); \
   \
  OGLFloat det=Det3(u[0], v[0], w[0], u[1], v[1], w[1], u[2], v[2], w[2]); \
  OGLFloat k=Det3(e[0], v[0], w[0], e[1], v[1], w[1], e[2], v[2], w[2])/det; \
  OGLFloat l=Det3(u[0], e[0], w[0], u[1], e[1], w[1], u[2], e[2], w[2])/det; \
  OGLFloat m=Det3(u[0], v[0], e[0], u[1], v[1], e[1], u[2], v[2], e[2])/det; \
   \
  OGLFloat pa1=PI*dx/180; \
  OGLFloat pa2=atan2(m, l) - PI*dy/180; \
  OGLFloat sina1=sin(pa1), cosa1=cos(pa1); \
  OGLFloat sina2=sin(pa2), cosa2=cos(pa2); \
   \
  if (cosa2 > 0) return;

void Viewport::camRotateCenter(double dx, double dy) {
  if (type != vpAxo && type != vpPersp) return;
  PREPARE_ROTATION;
  center[0]=eye[0] + oldLen*(w[0]*sina2 + cosa2*(v[0]*cosa1 + u[0]*sina1));
  center[1]=eye[1] + oldLen*(w[1]*sina2 + cosa2*(v[1]*cosa1 + u[1]*sina1));
  center[2]=eye[2] + oldLen*(w[2]*sina2 + cosa2*(v[2]*cosa1 + u[2]*sina1));
}

void Viewport::camRotateEye(double dx, double dy) {
  if (type != vpAxo && type != vpPersp) return;
  PREPARE_ROTATION;
  eye[0]=center[0] - oldLen*(w[0]*sina2 + cosa2*(v[0]*cosa1 + u[0]*sina1));
  eye[1]=center[1] - oldLen*(w[1]*sina2 + cosa2*(v[1]*cosa1 + u[1]*sina1));
  eye[2]=center[2] - oldLen*(w[2]*sina2 + cosa2*(v[2]*cosa1 + u[2]*sina1));
}

void Viewport::camScale(double dx, double dy) {
  OGLFloat pom=((fabs(dx)>fabs(dy))?-dx:dy);
  pom=(pom < 0.0)?(1/(1-pom/50)):1+pom/50;
  if (type == vpPersp) {
    if (fabs(dx) > fabs(dy)) {
      if ((fov*=pom) > 180.0) fov=180.0;
    } else {
      OGLVector3 d(center[0] - eye[0], center[1] - eye[1], center[2] - eye[2]);
      d.selfNormalize();
      center[0] += dy*d[0];
      center[1] += dy*d[1];
      center[2] += dy*d[2];
      eye[0] += dy*d[0];
      eye[1] += dy*d[1];
      eye[2] += dy*d[2];
    }
  } else scale*=pom;
}

void Viewport::camTranslate(double dx, double dy) {
  OGLVector3 dist(center[0] - eye[0],
		  center[1] - eye[1],
		  center[2] - eye[2]);
  OGLVector3 right(Det2(dist[1], dist[2], up[1], up[2]),
		   -Det2(dist[0], dist[2], up[0], up[2]),
		   Det2(dist[0], dist[1], up[0], up[1]));
  right.selfNormalize();
  OGLVector3 up(Det2(right[1], right[2], dist[1], dist[2]),
		-Det2(right[0], right[2], dist[0], dist[2]),
		Det2(right[0], right[1], dist[0], dist[1]));
  up.selfNormalize();
  eye[0]-=(dx*right[0] + dy*up[0])/scale;
  eye[1]-=(dx*right[1] + dy*up[1])/scale;
  eye[2]-=(dx*right[2] + dy*up[2])/scale;
  center[0]-=(dx*right[0] + dy*up[0])/scale;
  center[1]-=(dx*right[1] + dy*up[1])/scale;
  center[2]-=(dx*right[2] + dy*up[2])/scale;
}

void Viewport::redraw() {
  startRender();
  
  if (drawGridFlag) drawGrid();
  if (drawCenterFlag) drawCenter();
  if (drawCamerasFlag) drawCameras();
  if (drawLightsFlag) drawLights();
  if (drawBoundsFlag) drawBounds();
  else {
    if (drawObjectsFlag) drawObjects();
    if (drawNormalsFlag) drawNormals();
  }

  endRender();
}



GLViewport::GLViewport(int _number, viewportType _type, viewportRenderMode _mode):
  Viewport(_number, _type, _mode) {
  
  bgColor[0]=ViewBgRed; bgColor[1]=ViewBgGreen; bgColor[2]=ViewBgBlue;

  boundsDisplayList=0;
  invalidateDisplayLists=false;
}

void GLViewport::setRenderMode(viewportRenderMode newMode) {
  invalidateDisplayLists=true;
  Viewport::setRenderMode(newMode);
}

void GLViewport::redraw() {
  Viewport::redraw();
  invalidateDisplayLists=false;
}

void GLViewport::setLighting() {
  GLfloat lightPos[]={eye[0] - center[0], eye[1] - center[1], eye[2] - center[2], 0.0};
  GLfloat ambient[]={0.1, 0.1, 0.1, 1.0};
  glLightfv(GL_LIGHT0, GL_POSITION, lightPos);
  glLightfv(GL_LIGHT0, GL_AMBIENT, ambient);
  glEnable(GL_LIGHT0);  
}

void GLViewport::startRender() {
  glViewport(rcX, rcY, rcW, rcH);
  glDrawBuffer(GL_BACK);

  // Clear viewport
  glClearColor(bgColor[0], bgColor[1], bgColor[2], 1.0);
  glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);

  // Setup and remember (for gluProject and gluUnproject routines) modelview matrix
  glMatrixMode(GL_MODELVIEW);
  glLoadIdentity();
  glScaled(scale, scale, scale);
  gluLookAt(eye[0], eye[1], eye[2],
	    center[0], center[1], center[2],
	    up[0], up[1], up[2]);
  glScaled(1.0, 1.0, zInvert);
  glGetDoublev(GL_MODELVIEW_MATRIX, modelviewMatrix);

  // Setup and remember projection matrix
  glMatrixMode(GL_PROJECTION);
  glLoadIdentity();
  if (type != vpPersp) {
    glOrtho(-(GLdouble)rcW/2.0, (GLdouble)rcW/2.0,
	    -(GLdouble)rcH/2.0, (GLdouble)rcH/2.0,
	    -16384.0, 16384.0);
  } else {
    OGLVector3 d(center[0] - eye[0], center[1] - eye[1], center[2] - eye[2]);
    OGLFloat pl=d.len();
    glScaled((GLdouble)rcH/(GLdouble)rcW, 1.0, 1.0);
    gluPerspective(fov, 1.0, pl/100, 100*pl);
  }
  glGetDoublev(GL_PROJECTION_MATRIX, projectionMatrix);
}

void GLViewport::drawGrid() {
  OGLFloat gridStep=10.0;
  if (gridStep*scale < 5.0) return;

  if (mode == vpWireframe) glDisable(GL_DEPTH_TEST);
  else glEnable(GL_DEPTH_TEST);
  glDisable(GL_LIGHTING);
  glDisable(GL_POLYGON_OFFSET_FILL);
  glDisable(GL_NORMALIZE);

  glLineWidth(1.0);

  OGLFloat xStart = -((int)(((OGLFloat)rcW/(2.0*scale))/gridStep)+2)*gridStep;
  OGLFloat xEnd = ((int)(((OGLFloat)rcW/(2.0*scale))/gridStep)+2)*gridStep;
  OGLFloat yStart = -((int)(((OGLFloat)rcH/(2.0*scale))/gridStep)+2)*gridStep;
  OGLFloat yEnd = ((int)(((OGLFloat)rcH/(2.0*scale))/gridStep)+2)*gridStep;
  glColor3f(0.7, 0.7, 0.7);
  glEnable(GL_DEPTH_TEST);
  switch (type) {
    case vpTop:
      glBegin(GL_POINTS);
      xStart+=gridStep*((int)(eye[0]/gridStep));
      xEnd+=gridStep*((int)(eye[0]/gridStep));
      yStart-=gridStep*((int)(eye[2]/gridStep));
      yEnd-=gridStep*((int)(eye[2]/gridStep));
      for (OGLFloat y=yStart; y <= yEnd; y+=gridStep)
	for (OGLFloat x=xStart; x <= xEnd; x+=gridStep)
	  glVertex3d(x, 0.0, y);      
      glEnd();
      glBegin(GL_LINES);      
        glVertex3d(xStart, 0, 0);
	glVertex3d(xEnd, 0, 0);
	glVertex3d(0, 0, yStart);
	glVertex3d(0, 0, yEnd);
      glEnd();
      break;
    case vpFront:
      glBegin(GL_POINTS);
      xStart+=gridStep*((int)(eye[0]/gridStep));
      xEnd+=gridStep*((int)(eye[0]/gridStep));
      yStart+=gridStep*((int)(eye[1]/gridStep));
      yEnd+=gridStep*((int)(eye[1]/gridStep));
      for (OGLFloat y=yStart; y <= yEnd; y+=gridStep)
	for (OGLFloat x=xStart; x <= xEnd; x+=gridStep)
	  glVertex3d(x, y, 0.0);      
      glEnd();
      glBegin(GL_LINES);      
        glVertex3d(xStart, 0, 0);
	glVertex3d(xEnd, 0, 0);
	glVertex3d(0, yStart, 0);
	glVertex3d(0, yEnd, 0);
      glEnd();
      break;
    case vpRight:
      glBegin(GL_POINTS);
      xStart-=gridStep*((int)(eye[2]/gridStep));
      xEnd-=gridStep*((int)(eye[2]/gridStep));
      yStart+=gridStep*((int)(eye[1]/gridStep));
      yEnd+=gridStep*((int)(eye[1]/gridStep));
      for (OGLFloat y=yStart; y <= yEnd; y+=gridStep)
	for (OGLFloat x=xStart; x <= xEnd; x+=gridStep)
	  glVertex3d(0.0, y, x);      
      glEnd();
      glBegin(GL_LINES);      
        glVertex3d(0, 0, xStart);
	glVertex3d(0, 0, xEnd);
	glVertex3d(0, yStart, 0);
	glVertex3d(0, yEnd, 0);
      glEnd();
      break;
    case vpAxo:
    case vpPersp:
      OGLFloat px=((int)(center[0]/gridStep))*gridStep;
      OGLFloat py=((int)(center[2]/gridStep))*gridStep;
      OGLFloat pz=((int)(center[1]/gridStep))*gridStep;
      glBegin(GL_POINTS);
      for (OGLFloat y=-50; y <= 50; y++)
	for (OGLFloat x=-50; x <= 50; x++)
	  glVertex3d(px + x*gridStep, py + y*gridStep, 0);
      glEnd();
      glShadeModel(GL_SMOOTH);
      glLineWidth(2.0);
      glBegin(GL_LINES);
      if (center[2] >= -50*gridStep && center[2] <= 50*gridStep) {
	glColor3f(0.75, 0.75, 0.75);
	glVertex3d(px - 50*gridStep, 0, 0);
	glColor3f(0.75, 0.30, 0.30);
	glVertex3d(px + 50*gridStep, 0, 0);
      }
      if (center[0] >= -50*gridStep && center[0] <= 50*gridStep) {
	glColor3f(0.75, 0.75, 0.75);
	glVertex3d(0, py-50*gridStep, 0);
	glColor3f(0.30, 0.75, 0.30);
	glVertex3d(0, py+50*gridStep, 0);
      }
      if (center[1] >= -50*gridStep && center[1] <= 50*gridStep) {
	glColor3f(0.75, 0.75, 0.75);
	glVertex3d(0, 0, pz-50*gridStep);
	glColor3f(0.30, 0.30, 0.75);
	glVertex3d(0, 0, pz+50*gridStep);
      }
      glEnd();
      glShadeModel(GL_FLAT);
      glLineWidth(1.0);
      break;
  }
}

void GLViewport::drawCenter() {
  if (type != vpAxo && type != vpPersp) return;

  if (mode == vpWireframe) glDisable(GL_DEPTH_TEST);
  else glEnable(GL_DEPTH_TEST);
  glDisable(GL_LIGHTING);
  glDisable(GL_POLYGON_OFFSET_FILL);
  glDisable(GL_NORMALIZE);

  glLineWidth(1.0);

  glColor3f(0.7, 0.7, 0.7);
  glLineWidth(2.0);
  glBegin(GL_LINES);
    glVertex3d(center[0]-10, center[1], center[2]);
    glVertex3d(center[0]+10, center[1], center[2]);
    glVertex3d(center[0], center[1]-10, center[2]);
    glVertex3d(center[0], center[1]+10, center[2]);
    glVertex3d(center[0], center[1], center[2]-10);
    glVertex3d(center[0], center[1], center[2]+10);
  glEnd();
  glColor3f(0.6, 0.6, 0.6);
  glLineWidth(1.0);
  glBegin(GL_LINES);
    glVertex3d(center[0], center[1], center[2]);
    glVertex3d(center[0], center[1], 0.0);
    glVertex3d(center[0]-7, center[1], 0.0);
    glVertex3d(center[0]+7, center[1], 0.0);
    glVertex3d(center[0], center[1]-7, 0.0);
    glVertex3d(center[0], center[1]+7, 0.0);
  glEnd();
}

void GLViewport::drawCameras() {
}

void GLViewport::drawLights() {
}

void GLViewport::drawObjects() {
  switch (mode) {
    case vpWireframe:
      glDisable(GL_DEPTH_TEST);
      glDisable(GL_LIGHTING);
      glDisable(GL_POLYGON_OFFSET_FILL);
      glDisable(GL_NORMALIZE);
      glLineWidth(1.0);

      break;
    case vpHiddenline:
      glEnable(GL_DEPTH_TEST);
      glDisable(GL_LIGHTING);
      glDisable(GL_NORMALIZE);
      
      glShadeModel(GL_FLAT);

      //      glEnable(GL_POLYGON_OFFSET_FILL);
      //      glPolygonOffset(1.0, 0.5);
      
      break;
    case vpFlat:
      glEnable(GL_DEPTH_TEST);
      glEnable(GL_LIGHTING);
      glDisable(GL_POLYGON_OFFSET_FILL);
      glEnable(GL_NORMALIZE);

      glPolygonMode(GL_FRONT_AND_BACK, GL_FILL);
      glShadeModel(GL_FLAT);
      glDepthFunc(GL_LEQUAL);

      setLighting();

      break;
    case vpSmooth:
      glEnable(GL_DEPTH_TEST);
      glEnable(GL_LIGHTING);
      glDisable(GL_POLYGON_OFFSET_FILL);
      glEnable(GL_NORMALIZE);
      
      glPolygonMode(GL_FRONT_AND_BACK, GL_FILL);
      glShadeModel(GL_SMOOTH);
      glDepthFunc(GL_LEQUAL);

      setLighting();

      break;
    default:
      cerr << "GLViewport::drawObjects(): Unimplemented render mode" << endl;
  }
  
  glMatrixMode(GL_MODELVIEW);
}

void GLViewport::drawBounds() {
  glDisable(GL_DEPTH_TEST);
  glDisable(GL_LIGHTING);
  glDisable(GL_POLYGON_OFFSET_FILL);
  glDisable(GL_NORMALIZE);

  if (invalidateDisplayLists) {
    if (boundsDisplayList) glDeleteLists(boundsDisplayList, 1);
    boundsDisplayList=0;
  }

  if (boundsDisplayList) {
    glCallList(boundsDisplayList);
    return;
  }

  if (!(boundsDisplayList=glGenLists(1))) cerr << "GLViewport::drawBounds(): Cannost allocate display list" << endl;
  glNewList(boundsDisplayList, GL_COMPILE_AND_EXECUTE);
  
  glEndList();
}

void GLViewport::drawNormals() {
  //  glDisable(GL_LIGHTING);
  //  glDisable(GL_POLYGON_OFFSET_FILL);
  //  glDisable(GL_NORMALIZE);
}


GLDrawable::GLDrawable() {
  displayList=0;
}

GLDrawable::~GLDrawable() {
  invalidateDisplayList();
}

void GLDrawable::invalidateDisplayList() {
  if (displayList) glDeleteLists(displayList, 0);
  displayList=0;
}

void GLDrawable::drawToGLViewport(GLViewport *view, bool invalidateList) {
  if (invalidateList) invalidateDisplayList();

  if (displayList) {
    glCallList(displayList);
    return;
  }

  if (!(displayList=glGenLists(1))) cerr << "GLDrawable::drawToGLViewport(...): Cannot allocate display list" << endl;
  glNewList(displayList, GL_COMPILE_AND_EXECUTE);
  glDraw(view);
  glEndList();
}

/*--- End of file ---*/
